/* File CTRLpar.h; TRANSIENT V4.3; 30 October 2023 */
/* (C) 1995-2008,2011,2015,2017,2021,2023 Miroslav Kolar */

// Data for control input parameter declarations, output and short description.
// Checks of the bounds of some of these parameters are in  InOut.c  after the line 813,
// below the comment * Check some obvious bounds: *.

IN_PAR(steady,int,0,%d,
 0 - The main purpose of this program - the time-dependent (transient)\n\t
  solution of coupled reaction-diffusion equations in one spatial variable;\n\t
  all control parameters (as listed by ?c-) are used.\n\040\040
 1 - Only a steady state solution is obtained using the same evalf() function\n\t
  as for the transient solution\054 provided that PDM\047s evalf consistently uses\n\t
  global variable 'tStepInv' (tStepInv = 1/tStep) to calculate the contribution\n\t
  of discretized time derivatives of unknown variables to F[k] and B[k][k]; then\n\t
  simply setting tStepInv=0 eliminates all time derivative terms. Time is set to\n\t
  t=tStart (in case there are any time dependent reaction terms\054 but then\n\t
  the notion of a steady state may be meaningless).\n\t
  Only the following control parameters (may) have effect during steady-state run:\n\t
  steady\054 TOL\054 TOLC\054 sepProf(=-1)\054 echoInit\054 init_from_file\054 cDiffMfExcl\054 tStart.\n\040\040
 2 - First a steady state is obtained\054 and then used in the same run as the initial\n\t
  state for a transient solution (e.g.\054 to check this steady state stability).)

IN_PAR(monitor,int,0,%d,The amount of diagnostic on-screen output for a transient run:\n\040
 -1 - Only the value of t is printed at all \042pr_Other\042 output times.\n\040\040
  0 - In addition to the above\054 the value of cDiffM is printed at regular\n\t
 \042pr_Other\042 output times.\n\040\040
  1 - Values of t\054 cDiffM=(MAX over j and e of |c[j][e]-c_o[j][e]|)/cAVmax;\n\t
 where cAVmax=(MAX over j and e of |c[j][e]| and |c_o[j][e]|)\054\n\t\
 tStep\054 # of attempts\054 and total # of runband Newton-Raphson (NR) iterations\n\t
 in all attempts are printed after EACH time step\n\t
 (here e runs only through the "non-excluded" equations).\n\040\040
  2 - During EACH time step a line with iter #\054 (fresidT)\054 fresidM\054 node and\n\t
 equ # where fresidM occurs\054 (cresidT)\054 cresidM\054 and the corresponding\n\t
 node and equ #\054 are printed after each iteration. Then a line which\n\t
 starts with t\054 cDiffM\054 tStep\054 Scale = next_tStep/tStep ratio.\n\t
 Also some details about repeated attempts are given.\n\040\040
  3 - In addition to 2\054 another line is inserted giving where cDiffM occurs and the\054\n\t
 values of (MAX of |c_o|) and (MAX of |c|)\054 both over all nodes j and "non-excluded"\n\t
 equations e. If non-empty cDiffMfExcl is set (see transien.h)\054 one more line is\n\t
 added with cDiffMExcl and the respective max of |c_o|\054 |c|\054 for equations\n\t
 from the cDiffMfExcl list.\n\040\040
 10\054 20\054 30 - Same as 1\054 2\054 3\054 respectively\054 except that there is no pause at the\n\t
 start of a run allowing to read a brief explanation of the output to follow.\n\040\040
Except for monitor=-1\054 the (last) line continues with the cumulative numbers of\n\040\040\040\040
time steps\054 attempts (calls to runband)\054 and NR iterations (incl. those in\n\040\040\040\040
the unsuccessful attempts).\n\040\040
For monitor>0\054 it ends with the value of cDiffM extrapolated to the time\n\040\040\040\040
interval between two consecutive profile output times (to give an estimate\n\040\040\040\040
of max relative change between the consecutive outputted profiles:\n\040\040\040\040
e.g.\054 0.01 ~ change by about 1 in the 3rd significant digit; when these estimates are\n\040\040\040\040
much larger than the real differences\054 it indicates numerical noise or oscillations).\n\040\040
For monitor>0\054 output into the two main output files is flagged by additional\n\040\040\040\040
lines containing <outOTH> or <out[ n]>.)

IN_PAR(CmntC,char,'#',%c,Character with a dual (complementary\054 contrary) role:\n\040
 - In all output files it marks the beginning of comment lines (lines that would\n\040\040
  not be plotted).\n\040
 - In the input file\054 or in the interactive input at the command prompt\054\n\040\040
  any number of its occurrences at the beginning of input lines are equivalent\n\040\040
  to white space. On the contrary\054 any character other than CmntC or a space can\n\040\040
  be used at the beginning of an input line to 'comment it out' (ignore it).\n\040\040
  Thus\054 CmntC may be reset in an input file repeatedly to switch on or off blocks\n\040\040
  of lines that are prefixed with different characters. Only the last\n\040\040
  value of CmntC read in from the input file is then used as the comment\n\040\040
  character in the output files.)

IN_PAR(TOL,double,1.,%g,Successful NR convergence condition:\n\t
 if cresidT < TOLC*TOL*(#equations-Nexcl)*#nodes*2e-16  OR  fresidT < TOL*#nodes*2e-16)

IN_PAR(TOLC,double,10.,%g,See help for TOL)

IN_PAR(Tau,double,0.5,%.10g,\042Implicitness\042 of the differencing scheme (in time;
 0 <= Tau <= 1):\n\t\t
 0\tfully implicit\n\t\t 0.5\tCrank-Nicholson\n\t\t 1\texplicit\n\040\040
 Preferably use Tau = 0.5; if there are problems\054 try Tau < 0.5;\n\040\040
 in some cases\054 Tau = 0 may be the best.)

IN_PAR(NIG,double,0.5,%.10g,New Initial Guess extrapolation factor (-0.5<=NIG<=1) for the next time step:\n\040\040
 c_New_Initial_Guess = c + NIG * Scale * (c - c_previous);\n\040\040
 where Scale = (t_new - t) / (t - t_previous)\054\n\040\040
 and t & c are the current time and the unknown variable value\054 respectively)

IN_PAR(nIterM,int,7,%d,Desired (≈ maximum) number of NR iterations in runband() in each attempt)

IN_PAR(tStart,double,0.,%.12g sec,Initial time (start of transient simulation))

IN_PAR(tEnd,double,36000.,%.12e sec,Final time (end of transient simulation))

IN_PAR(tStepInit,double,0.001,%.8g sec,Magnitude of the initial time step tStep)

IN_PAR(maxScale,double,1.1,%g,Maximum allowed ratio of two consecutive time steps\054\n\040\040
 i.e.\054 maximum of 'Scale' defined in the description of the control parameter NIG)

IN_PAR(tOutInit,double,1800.,%.12e sec,Initial time (tOut) for c profiles output)

IN_PAR(tOutMulF,double,1.4,%.10g,Rate of increase of the time interval between
 subsequent c profiles outputs:\n\040\040
 Next_tOut = tOut + (tOut - previous_tOut) * tOutMulF)

IN_PAR(nOutOTH,int,-20,%d,
pr_Other() is called at |nOutOTH|-1 regularly spaced time instants\n\040\040
 between 2 consecutive tOut (can be used for additional\054 time-dependent\n\040\040
 output\054 placed into the .tdep output file).\n\040\040
 If nOutOTH<0\054 pr_Other calls MAY NOT be exactly equidistant in time.)

IN_PAR(sepProf,int,1,%d,0 - Concentration profiles for all output times will be placed\n\t
  into a single output file.\n\040\040
 1 - Profiles for each time will be placed into a separate file.\n\040
-1 - No profiles file created. In addition\054 no .diag file created.)

IN_PAR(echoInit,int,0,%d,
Output of initial profiles (after possible reinitialization\054 see init_from_file):\n\040\040
 0 - Disabled\n\040\040
 1 - Initial profiles are written out into an output file with the extension 0)

IN_PAR(strict,int,-1,%d,
strict < 0: cDifMxf(t) is not used at all to determine the next tStep\054\n\t
  and its growth is limited only by nIterM.\n\040\040
 strict = 0: cDiffM does usually not exceed cDifMxf(t) too much.\n\040\040
 strict = 1: cDiffM <= cDifMxf(t) is strictly enforced.\n\t
  Here cDiffM is the maximum magnitude of the change in all non-excluded\n\t
 c[][]/cAVmax over the whole tStep (see the help for monitor=1); cresidM is\n\t
 the same quantity\054 but for a single NR iteration only.\n\t
 cDifMxf(t) is determined by input parameters cDifMxMode\054 cDifMxInit\054\n\t
 and if applicable by cDifMxPi (i=1,...,cDifMxMode-2) and cDifMxEnd.\n\t
 strict<0 gives the fastest run\054 and sometimes produces good enough results.)

IN_PAR(cDifMxMode,int,0,%d,
Type of cDifMxf(t)\054 i.e. of the interpolation between cDifMxInit and cDifMxEnd:\n\040\040
 0 - Max allowed change in c is constant and equal to cDifMxInit\n\040\040
 1 - Linear interpolation\n\040\040
 2 - Log-log linear: ln(cDifMxf) = linear function of ln(|t|)\n\040
>2 - Piecewise linear: In this case cDifMxMode = number of all time instants\n \040 \040 \040
between each consecutive two of which a linear interpolation of cDifMxf\n \040 \040 \040
will be done. This number includes the initial and final times\054 corresponding\n \040 \040 \040
to values cDifMxInit and cDifMxEnd. The remaining cDifMxMode-2 inner points\n \040 \040 \040
are entered as values of input parameters cDifMxP1\054 cDifMxP2\054 ...\054\n \040 \040 \040
cDifMxP{cDifMxMode-2}. Their values are the comma-separated\n \040 \040 \040
(no spaces allowed around the comma) pairs of time (in sec) and the\n \040 \040 \040
corresponding value of cDifMxf. Example:\n\t\t
 cDifMxInit = 1e-5\n\t\t
 cDifMxP3 = 1e5\0540.1\n\t\t
 cDifMxP1 = 10.\0541e-4\n\t\t
 cDifMxEnd = 1.\n\t\t
 cDifMxMode = 5\n\t\t
 cDifMxP2 = 1e3\0540.01\n \040 \040 \040
(Order of input lines does not matter\054 but times corresponding to\n \040 \040 \040
cDifMxP{i} must increase with i if tEnd>tStart\054\n \040 \040 \040
and decrease with i if tEnd<tStart.))

IN_PAR(cDifMxInit,double,1.,%.8g,Value of cDifMxf at t = tStart+sign(tEnd-tStart)*tStepInit)

/* cDifMxEnd moved to InOut.c, because not needed when cDifMxMode=0.

Altogether, these remaining "atypical" control input parameters are declared in InOut.c:
cDifMxEnd,
cDifMxP1, cDifMxP2, ... (look for cDifMxP[] and tcDifMxP[]);
cDiffMfExcl,
init_from_file
*/
