/* File DateTime.c; TRANSIENT V4.3; 26 July 2017 */
/* Copyright (C) 1992-2001,2011,2017 Miroslav Kolar */

/*
* DateAndTime() returns pointer to a string containing date and time of the
  instant of invocation. For example: Wednesday, 2 August 1992, 1:50:01 MDT.

* If the time-zone field is not desired as part of the file creation date
 (or if tzname[] is not properly defined), enable the following line:
#define NO_TIME_ZONE
* To get everything working in VMS, at least OpenVMS, one should compile
  with something like this: /CC/DECC/PREFIX_LIBRARY_ENTRIES=ALL_ENTRIES.
* If you are getting incorrect (Pacific) time zone on a PC, set properly
  the TZ environment variable.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "DateLen.h"

#define PE         (void)fprintf(stderr,

/* From InOut.c : */
extern void exitTR(int stat, char *descr) __attribute__ ((noreturn));

#if !defined(NO_TIME_ZONE) &&\
 (defined(__CYGWIN__) || defined(__CYGWIN32__) || defined(__BORLANDC__))
#define tzname _tzname
#endif

#define STRLEN(x)  ((x) == NULL ? 0 : (size_t)strlen(x))


void DateLenCheck(int line, int len)
//==================================
{
 if(len >= DATELEN) {
	PE" "__FILE__": line %d: DATELEN must be increased at least to %d!\n",
			line, len + 1);
	exitTR(-3,"DATELEN in DateLen.h must be increased");
 }
}


char *DateAndTime(void)
/*===================*/
{
int len;
static char dateTime[DATELEN];
struct tm *tms;
time_t seconds;
static char mon[12][10] = { "January","February","March","April","May",
			    "June","July","August","September",
                            "October","November","December" },
	    day[7][10] = { "Sunday","Monday","Tuesday","Wednesday",
			   "Thursday","Friday","Saturday" };

#if !defined(NO_TIME_ZONE) && (defined(__CYGWIN__) || defined(__CYGWIN32__))
char *c;
tzset();
if((c = strrchr(tzname[0],'S')) != NULL) {
	*++c = 'T';
	*++c = 0;
}
if(_daylight && (c = strrchr(tzname[1],'D')) != NULL) {
	*++c = 'T';
	*++c = 0;
}
#endif

(void)time(&seconds);
tms = localtime(&seconds);

dateTime[0] = 0;
len=snprintf(dateTime,DATELEN,"%s, %d %s %d, %d:%02d:%02d",
   day[tms->tm_wday], tms->tm_mday, mon[tms->tm_mon], 1900 + tms->tm_year,
   tms->tm_hour, tms->tm_min, tms->tm_sec);
if(len > 0) {
 DateLenCheck(__LINE__, len);
#ifndef NO_TIME_ZONE
#if defined(sparc) || defined(ultrix) || defined(__osf__)\
 || defined(SYSTEM_FIVE) || defined(sgi) || defined(__GNUC__)\
 || defined(vms) || defined(VMS)\
 || defined(MSDOS) || defined(__BORLANDC__) || defined(_MSC_VER)
 { int LEN;
   char *TZ;
#if defined(sparc) || defined(ultrix) || defined(__osf__)
   TZ = tms->tm_zone;
#else
   TZ = tms->tm_isdst ? tzname[1] : tzname[0];
#endif
   LEN = len + 1 + STRLEN(TZ);
   DateLenCheck(__LINE__, LEN);
   (void)sprintf(dateTime + len," %s", TZ);
 }
#endif
#endif
}
return(dateTime);
}


/* *********************************************************************

An alternative definition:

#include <stdio.h>
#include <time.h>
#include <string.h>

char *DateAndTime(void)
{
char *DaT, *nlPos;
time_t seconds;

(void)time(&seconds);
DaT = ctime(&seconds);
if((nlPos = strstr(DaT,"\n")) != NULL && nlPos != DaT) *nlPos = 0;
return(DaT);
}

***********

or if there are any problems with local time or you wish to switch off
the time-stamping altogether, just use the following:

#include <stdio.h>

char *DateAndTime(void)
{
return("(not available)");
}

*********************************************************************** */
