/* File demo/demo1/demo1.h; TRANSIENT V3.0; 5 June 2005 */
/* (C) 2001,2005 Miroslav Kolar */

IN_PAR(L1,double,0.5,%g m,Thickness of medium 1)
IN_PAR(L2,double,25.,%g m,Thickness of medium 2)
IN_PAR(interv1,int,75,%d,Number of intervals in "[0,L1]")
IN_PAR(interv2,int,45,%d,Number of intervals in "[L1,L1+L2]")
CHECK_PAR(interv1>0)
CHECK_PAR(interv2>0)
CHECK_PAR(L1>0)
CHECK_PAR(L2>0)

IN_STRPAR(frmt,10,"%.8g",C format for double - concentrations & corrosion current & potential)
/* A space will be added at the beginning -> max usable length=8: */
CHECK_PAR(strlen(frmt)<9)
/* CHECK_PAR(frmt[0]=='%') - did not print well; instead: */
CHECK_PAR(!strncmp(frmt,"%%",1))
/* Further tests for  frmt  in Initialize() */

IN_PAR(ZERO,double,1e-24,%g mol/m^3,O2 concentration will be reset to 0 if it is less than\n\040\040	the value of ZERO - to try to suppress numerical noise)
CHECK_PAR(ZERO>=0)

IN_PAR(onlyCuI,int,0,%d,0 - full model solved;\n\040\040\
  1 - only eqs. 1+2 for CuI;\n\040\040\
  2 - only eq. 1 (k5=0; c2 converted to c1 (in read_profiles));\n\040\040\
  If 1 or 2: using bounds to keep other c=0; Cmed=0)
CHECK_PAR(onlyCuI==0 || (onlyCuI==1||onlyCuI==2) && *init_from_file)

IN_PAR(D0,double,1.7e-9,%g m^2/s,Diffusion coefficient for O2)
IN_PAR(D1,double,5.5e-10,%g m^2/s,Diffusion coefficient for Cu(I))
IN_PAR(D3,double,6.0e-10,%g m^2/s,Diffusion coefficient for Cu(II))
IN_PAR(k5,double,0.1,%g 1/s,Reaction rate)
IN_PAR(km5,double,1.,%g 1/s,Reaction rate)
IN_PAR(k7,double,0.029,%g m^3/(mol.s),Reaction rate)
IN_PAR(k8,double,1e-8,%g m^3/(mol.s),Reaction rate)
IN_PAR(k9,double,1e-5,%g 1/s,Reaction rate)
IN_PAR(km9,double,1e-6,%g 1/s,Reaction rate)
IN_PAR(k11,double,2e-6,%g m^3/(mol.s),Reaction rate)
IN_PAR(km11,double,1e-6,%g 1/s,Reaction rate)
CHECK_PAR(D0>0)
CHECK_PAR(D1>0)
CHECK_PAR(D3>0)
CHECK_PAR(k5>=0.)
CHECK_PAR(km5>=0.)
CHECK_PAR(k7>=0.)
CHECK_PAR(k8>=0.)
CHECK_PAR(k9>=0.)
CHECK_PAR(km9>=0.)
CHECK_PAR(k11>=0.)
CHECK_PAR(km11>=0.)

IN_PAR(n0,int,4,%d,Number of electrons for O2 reduction)
IN_PAR(n1,int,1,%d,Number of electrons for Cu dissolution)
IN_PAR(Gamma,double,0.7,%g,Activity coefficient for CuCl2-)
IN_PAR(A1,double,0.2065,%g A^3/mol^2,Electrochemical constant)
IN_PAR(A2,double,13.7,%g A.m/mol,Electrochemical constant)
IN_PAR(A3,double,-0.105,%g V,Electrochemical constant (standard potential))
IN_PAR(A4,double,6.281e-4,%g m^2/A,Electrochemical constant)
CHECK_PAR(n0>0)
CHECK_PAR(n1>0)
CHECK_PAR(Gamma>0.)
CHECK_PAR(A1>=0.)
CHECK_PAR(A2>=0.)
CHECK_PAR(A3<=0.)
CHECK_PAR(A4>=0.)

IN_PAR(Cmed1,double,0.2,%g mol/m^3,Initial [O2] (c0) in medium 1)
IN_PAR(Cmed2,double,3.1e-4,%g mol/m^3,Initial [O2] (c0) in medium 2\n\040\040\
	RHS boundary [O2] for all times)
IN_PAR(c1sat,double,0.1,%g mol/m^3,Equilibrium (saturated) [Cu(I)])
IN_PAR(c3sat,double,1.67e-4,%g mol/m^3,Equilibrium (saturated) [Cu(II)])
IN_PAR(s2max,double,140.25,%g mol/m^3,Maximum c5 (rho * [sorbed Cu]) in medium 1)
CHECK_PAR(Cmed1>=0.)
CHECK_PAR(Cmed2>=0.)
CHECK_PAR(c1sat>=0.)
CHECK_PAR(c3sat>=0.)
CHECK_PAR(s2max>0.)

IN_PAR(tau1,double,0.1,%g,Tortuosity factor of solution-filled pores in medium 1)
IN_PAR(tau2,double,0.0029355,%g,Tortuosity factor of medium 2)
IN_PAR(eps1E,double,0.1,%g,Effective porosity for diffusion in medium 1)
IN_PAR(eps1A,double,0.2,%g,Accessible porosity to the solution in medium 1)
IN_PAR(eps2,double,0.003,%g,Porosity of medium 2)
CHECK_PAR(tau1>0.)
CHECK_PAR(tau2>0.)
CHECK_PAR(eps1E>0.&&eps1E<=1.)
CHECK_PAR(eps1A>0.&&eps1A<=1.)
CHECK_PAR(eps2>0.&&eps2<=1.)

IN_PAR(Rgas,double,8.314,%g J/(K.mol),Gas constant)
IN_PAR(Farad,double,96487.,%g C/mol,Faraday constant)
IN_PAR(Tabs,double,298.2,%g K,Absolute temperature)
CHECK_PAR(Tabs>0.)
CHECK_PAR(Rgas>0.)
CHECK_PAR(Farad>0.)


/* Positive real time is required: */
CHECK_PAR(tStart>=0.)
CHECK_PAR(tEnd-tStart>0.)
