// File demo/demo2/demo2.h; TRANSIENT V4.3; 31 Oct. 2023
// (C) 2001,2005,2023 Miroslav Kolar                 
                                                     
// Common header file for both demo2a and demo2b PDMs

IN_PAR(l1,double,10.,%g cm,Length of narrow inlet pipe)
IN_PAR(l2,double,50.,%g cm,Length of container)
IN_PAR(d1,double,0.6,%g cm,Diameter of narrow inlet pipe)
IN_PAR(d2,double,20.,%g cm,Diameter of container)

#if VERSN == 1
IN_PAR(interv,int,200,%d,No. of grid intervals in "[0,l1+l2]")
#else
IN_PAR(interv1,int,50,%d,No. of grid intervals in "[0,l1]")
IN_PAR(interv2,int,150,%d,No. of grid intervals in "[l1,l1+l2]")
#endif

IN_PAR(D0,double,0.634,%g cm^2/s,H2 diffusivity in air at T=0C and atm pressure)
IN_PAR(alfa,double,1.75,%g,Exponent in the dependence of H2 D0 on T)
IN_PAR(T,double,-10.,%g C,Operating temperature in Celsius degrees)
IN_PAR(G,double,2.57e-7,%g 1/s,H2 volume generation rate)

IN_PAR(s,double,1e-9,%g 1/s,Value of the Laplace Transform variable to compare\
\n\040\040	numerical results with the analytical ones -\
\n\040\040	only Laplace Transform of the analytical solution is available.)

IN_PAR(STEADYtol,double,1e-5,%g,Steady-state quality indicator: variation in concentration\n\040\040\
 at 3 interfaces at the end of the run should be less than STEADYtol)

#if VERSN == 1
IN_PAR(INTERFACE,int,1,%d,How interface between the two regions is handled:\n\040\040\
	0 - use mass balance equation & original l1 & l2;\n\040\040\
	1 - as 0 but adjust l1 & l2 so that a node lies on the interface;\n\040\040\
	2 - 2point formulas for flux in continuity condition & orig l1 & l2;\n\040\040\
	3 - the same as 2 but adjust l1 & l2 as in 1.)
#else
IN_PAR(INTERFACE,int,0,%d,How interface between the two regions is handled:\n\040\040\
	0;1 - use mass balance equation;\n\040\040\
	>=2 - 2point formulas for flux in continuity condition.)
#endif

IN_STRPAR(BCr,8,"massBal",Type of formula for the zero flux RHS b.c.:\n\040\040\
	massBal - full mass-balance equation for the last half interval;\n\040\040\
	3point - standard three-point discretization formula)

#if VERSN == 1
CHECK_PAR(interv>0)
#else
CHECK_PAR(interv1>0)
CHECK_PAR(interv2>0)
#endif
CHECK_PAR(l1>0.)
CHECK_PAR(l2>0.)
CHECK_PAR(d1>0.)
CHECK_PAR(d2>0.)
CHECK_PAR(D0>0.)
CHECK_PAR(alfa>0.)
CHECK_PAR(G>=0.)
CHECK_PAR((T+273.15)>0.)
CHECK_PAR(s>0.)
#if VERSN == 1
CHECK_PAR(INTERFACE>=0 && INTERFACE<=3)
#else
CHECK_PAR(INTERFACE>=0)
#endif
CHECK_PAR(!strcmp(BCr,"massBal") || !strcmp(BCr,"3point"))

CHECK_PAR(STEADYtol>0.)

/* Needed for Laplace transform: */
CHECK_PAR(tStart==0.)
CHECK_PAR(tEnd>0.)
