/* File demo/demo2/demo2a.c; TRANSIENT V4.3;	31 Oct. 2023	*/
/* (C) 1995,2001,2005 Miroslav Kolar */

/* ============================================================ */
/* Diffusion of gas (e.g., H_2) generated inside a cylindrical	*/
/* container at a constant rate out of this container through	*/
/* a (narrow) opening (fill pipe).				*/
/* Demonstration module No. 2a for TRANSIENT by M. Kolar, 1995  */
/* ============================================================ */

/* This version (a) uses a single equidistant grid that spans	*/
/* both regions --> there may not be a node exactly on their	*/
/* interface.							*/

#define VERSN 1

// The PDM specific input parameters are defined in demo2.h :
#define PDM_H "demo/demo2/demo2.h"
	/* path of demo2.h relative to the TRANSIENT directory */
#include "../../transien.h"
	/* >>>> READ the information in transien.h of TRANSIENT <<<< */

/* Other global variables: */
static double
	*F, *A, *B, *D,
	a, aNew, bNew,
	aP, aM, aPNew, aMNew, aPMNew,
	New, g1;
double
	A1, A2, delta, h,
	*C, *C_O, D_T,
	*k, *ki,
	c0, cl1, cl1l2, c0p, cl1p, cl1l2p; /* Used to store the most
		recent and the previous conc'n values to test for steady
		state for the LT comparision; see also STEADYtol */

int
	interface, bcr;


void generate_grid(void)
/*====================*/
{
nnodes = interv + 1;
h = (l1 + l2) / (double)interv;

interface = l1/h + 0.5;
delta = l1 - interface * h;
if(INTERFACE & 1) {
 l1 -= delta;
 l2 += delta;
 delta = 0.;
 PE"l1 adjusted to %g cm, l2 adjusted to %g cm\n",l1,l2);
 Pp"%c   l1 adjusted to %g cm, l2 adjusted to %g cm\n",CmntC,l1,l2);
}

Pp"%c Other calculated values:\n",CmntC);
Pp"%c -----------------------\n",CmntC);
Pp"%c   x step = %g cm\n",CmntC,h);
Pp"%c   Interface from node distance: delta = %g cm\n",CmntC,delta);
}


void initNL3BANDV(double ***LBp, double ***UBp, double *Fp,
	double*Ap, double*Bp, double*Dp)
/*=======================================================*/
{
int j;
static double *LB, Lower[2];

*LBp = &LB;
Lower[0] = 1.;  /* Bound type: constant */
Lower[1] = 0.;  /* Bound value */
LB = Lower;
/* No upper bound set */

F= Fp;
A= Ap;
B= Bp;
D= Dp;

}


void auxiliary_parameters_for_evalf(void)
/*=====================================*/
{
double denom, aux1, aux2;

New = 1. - Tau;

D_T = D0 * pow((T + 273.15)/273.15, alfa);
A1 = 0.25 * M_PI * d1 * d1; /* cross-section of the inlet */
A2 = 0.25 * M_PI * d2 * d2; /* cross-section of the container */

Pp"%c   D_T = %g cm^2/s\n",CmntC,D_T);
Pp"%c   A1 = %g cm^2\n",CmntC,A1);
Pp"%c   A2 = %g cm^2\n%c\n",CmntC,A2,CmntC);
(void)fflush(par_out);

a = D_T / (h * h);
aNew = a * New;
bNew = -2. * aNew;

/* For the interface between the two cylinders the 
   following quantities are needed: */
aux1 = A1 * (0.5 * h + delta);
aux2 = A2 * (0.5 * h - delta);
denom = aux1 + aux2;
g1 = G * aux2 / denom;
aP = D_T * A2 / (denom * h);
aM = D_T * A1 / (h * denom);
aPNew = aP * New;
aMNew = aM * New;
aPMNew = -(aPNew + aMNew);
}


void evalf(int j)
/*=============*/
{
/* Remember, that in the calling procedure, X=A and Y=D.  */
static double  cBar0M, cBarP0, B0;

if(j==0){

  /* Assume that the first call to evalf() is with j=0 and
     on each subsequent call j is increased by 1 (THIS IS NOT
     CHECKED HERE!).
     So first do some initializations needed later for j > 0:
  */
	B0 = bNew - tStepInv; 
	cBarP0 = Tau * (C_O[1]-C_O[0]) + New * (C[1]-C[0]);

  /* Now do the x=0 (zero concentration) boundary conditions: */
    F[0]= C[0];
    B[0]= 1;

} else if(j<interv) {

 /* First update cBar's: */
 cBar0M = cBarP0;
 cBarP0 = Tau * (C_O[j+1] - C_O[j]) + New * (C[j+1] - C[j]);

 if(j<interface) { /* Narrow inlet */
	F[0] = a * (cBarP0 - cBar0M) + (C_O[j] - C[j]) * tStepInv;
	D[0] = A[0] = aNew;
	B[0] = B0;
 }

 else if(j == interface) {
	if(INTERFACE < 2) {
	F[0] = aP * cBarP0 - aM * cBar0M + g1
		+ (C_O[j] - C[j]) * tStepInv;
	A[0] = aMNew;
	B[0] = aPMNew - tStepInv;
	D[0] = aPNew;
	} else {
 /* The following straightforward application of flux continuity at
    the interface gives somewhat worse results than the above
    use of mass-balance equation for the region of the interface node.
 */
    	F[0] = A2 * (C[j+1] - C[j]) - A1 * (C[j] - C[j-1]);
	A[0] = A1;
	B[0] = -(A1 + A2);
	D[0] = A2;
	}
 }

 else { /* Container interior */
	F[0] = a * (cBarP0 - cBar0M) + G + (C_O[j] - C[j]) * tStepInv;
	D[0] = A[0] = aNew;
	B[0] = B0;
 }

} else { /* j=interv: zero-flux b.c. */

    if(bcr) { // mass balance
	F[0]= G - 2.*a*cBarP0 + (C_O[j] - C[j]) * tStepInv;
	A[0]= -bNew;
	B[0]= B0;
    } else { // 3-point
	F[0]= 3 * C[j] - 4 * C[j-1] + C[j-2];
	B[0]= 3;
	A[0]= -4;
	D[0]= 1; /* Here D plays the role of Y */
    }

}

}


void pr_profiles(int MaxIter)
/*=========================*/
{
int j, inlet_last = interface;
static int count = 0; /* For steady=2, count enables to take different
	action below for steady state calculation proper and for the
	subsequent transient calculation */

if(delta < 0.) inlet_last--;  /* inlet_last is the # of the last node
	that is inside the inlet; needed only for the purpose of
	comparison with analytical results */

if(steady && !count++) {
 double C1, C2 = A2 * l2 / A1, C3 = G/D_T;
 C1 = C3 * C2 * h;
 C2 *= l1 * C3;
 PR"%c\t\tSteady State\n%c\n", CmntC, CmntC);
 PR"%c No. of iterations done = %d\n", CmntC, MaxIter);
 report_max_NR_error(out, CmntC);
 PR"%c  x (cm)      [H_2]    [Analytical steady state]\n", CmntC);
 for(j= 0;j<=inlet_last;j++) PR" %f  %e  %e\n", j * h, C_O[j], C1 * j);
 for( ;j<nnodes;j++) {
	C1 = j * h - l1;
	PR" %f  %e  %e\n", j * h, C_O[j], C3 * C1 * (l2 - 0.5 * C1) + C2);
 }
} else {
 PR"#Time = %e sec = %e hours = %e days\n", t, t/3600., t/86400.);
 PR"#Last tStep = %e sec = %e hours\n", tStepLast, tStepLast/3600.);
 PR"#Max. # of iterations done since last profile = %d\n",MaxIter);
 report_max_NR_error(out, CmntC);
 PR"%c  x (cm)      [H_2]\n", CmntC);
 for(j= 0;j<nnodes;j++) PR" %f  %e\n", j * h, C_O[j]);
 c0p= c0; c0= C_O[0];
 cl1p= cl1; cl1= C_O[interface];
 cl1l2p= cl1l2; cl1l2= C_O[j];
}
(void)fflush(out);
}
