/* File demo/demo2/demo2b.c; TRANSIENT V4.3;	31 Oct. 2023	*/
/* (C) 1995,2001,2005 Miroslav Kolar */

/* ============================================================ */
/* Diffusion of gas (e.g., H_2) generated inside a cylindrical	*/
/* container at a constant rate out of this container through	*/
/* a (narrow) opening (fill pipe).				*/
/* Demonstration module No. 2b for TRANSIENT by M. Kolar, 1995  */
/* ============================================================ */

/* This version (b) uses different equidistant grids in the	*/
/* two regions --> a node is always on their interface		*/

#define VERSN 2

// The PDM specific input parameters are defined in demo2.h :
#define PDM_H "demo/demo2/demo2.h"
	/* path of demo2.h relative to the TRANSIENT directory */
#include "../../transien.h"
	/* >>>> READ the information in transien.h of TRANSIENT <<<< */

/* Other global variables: */
static double
	*F, *A, *B, *D,
	a1, a1New, b1New,
	a2, a2New, b2New,
	aP, aM, aPNew, aMNew, aPMNew,
	New, g1;
double
	A1, A2, h1, h2,
	*C, *C_O, D_T,
	*k, *ki,
	c0, cl1, cl1l2, c0p, cl1p, cl1l2p; /* Used to store the most
		recent and the previous conc'n values to test for steady
		state for the LT comparision; see also STEADYtol */

static int interv;
int bcr;


void generate_grid(void)
/*====================*/
{
interv = interv1 + interv2;
nnodes = interv + 1;
h1 = l1 / (double)interv1;
h2 = l2 / (double)interv2;

Pp"%c Other calculated values:\n",CmntC);
Pp"%c -----------------------\n",CmntC);
Pp"%c   x step in [0,l1] = %g cm\n",CmntC,h1);
Pp"%c   x step in [l1,l1+l2] = %g cm\n",CmntC,h2);
}


void initNL3BANDV(double ***LBp, double ***UBp, double *Fp,
	double*Ap, double*Bp, double*Dp)
/*=======================================================*/
{
int j;
static double *UB, *LB, NoBound = 0., Lower[2];

*LBp = &LB;
*UBp = &UB;
Lower[0] = 1.;  /* Bound type: constant */
Lower[1] = 0.;  /* Bound value */
LB = Lower;
UB = &NoBound; /* Another way to set no upper bound */

F= Fp;
A= Ap;
B= Bp;
D= Dp;

}


void auxiliary_parameters_for_evalf(void)
/*=====================================*/
{
double denom, aux1, aux2;

New = 1. - Tau;

D_T = D0 * pow((T + 273.15)/273.15, alfa);
A1 = 0.25 * M_PI * d1 * d1; /* cross-section of the inlet */
A2 = 0.25 * M_PI * d2 * d2; /* cross-section of the container */

Pp"%c   D_T = %g cm^2/s\n",CmntC,D_T);
Pp"%c   A1 = %g cm^2\n",CmntC,A1);
Pp"%c   A2 = %g cm^2\n%c\n",CmntC,A2,CmntC);
(void)fflush(par_out);

a1 = D_T / (h1 * h1);
a1New = a1 * New;
b1New = -2. * a1New;
a2 = D_T / (h2 * h2);
a2New = a2 * New;
b2New = -2. * a2New;

/* For the interface between the two cylinders the 
   following quantities are needed: */
aux1 = 0.5 * A1 * h1;
aux2 = 0.5 * A2 * h2;
denom = aux1 + aux2;
g1 = G * aux2 / denom;
aP = D_T * A2 / (h2 * denom);
aM = D_T * A1 / (h1 * denom);
aPNew = aP * New;
aMNew = aM * New;
aPMNew = -(aPNew + aMNew);
}


void evalf(int j)
/*=============*/
{
/* Remember, that in the calling procedure, X=A and Y=D.  */
static double  cBar0M, cBarP0, B0_1, B0_2;

if(j==0){

  /* Assume that the first call to evalf() is with j=0 and
     on each subsequent call j is increased by 1 (THIS IS NOT
     CHECKED HERE!).
     So first do some initializations needed later for j > 0:
  */
	B0_1 = b1New - tStepInv; 
	B0_2 = b2New - tStepInv; 
	cBarP0 = Tau * (C_O[1]-C_O[0]) + New * (C[1]-C[0]);

  /* Now do the x=0 (zero concentration) boundary conditions: */
    F[0]= C[0];
    B[0]= 1;

} else if(j<interv) {

 /* First update cBar's: */
 cBar0M = cBarP0;
 cBarP0 = Tau * (C_O[j+1] - C_O[j]) + New * (C[j+1] - C[j]);

 if(j<interv1) { /* Narrow inlet */
	F[0] = a1 * (cBarP0 - cBar0M) + (C_O[j] - C[j]) * tStepInv;
	D[0] = A[0] = a1New;
	B[0] = B0_1;
 }

 else if(j == interv1) {
	if(INTERFACE < 2) {
	F[0] = aP * cBarP0 - aM * cBar0M + g1
		+ (C_O[j] - C[j]) * tStepInv;
	A[0] = aMNew;
	B[0] = aPMNew - tStepInv;
	D[0] = aPNew;
	} else {
 /* The following straightforward application of flux continuity at
    the interface gives somewhat worse results than the above
    use of mass-balance equation for the region of the interface node.
 */
	A[0] = A1 / h1; /* For faster execution, one could put A1/h1  */
	D[0] = A2 / h2; /* and A2/h2 into new variables in Initialize */
    	F[0] = D[0] * (C[j+1] - C[j]) - A[0] * (C[j] - C[j-1]);
	B[0] = -(A[0] + D[0]);
	}
 }

 else { /* Container interior */
	F[0] = a2 * (cBarP0 - cBar0M) + G + (C_O[j] - C[j]) * tStepInv;
	D[0] = A[0] = a2New;
	B[0] = B0_2;
 }

} else { /* j=interv: zero-flux b.c. */

    if(bcr) { // mass balance
	F[0]= G - 2.*a2*cBarP0 + (C_O[j] - C[j]) * tStepInv;
	A[0]= -b2New;
	B[0]= B0_2;
    } else { // 3-point
	F[0]= 3 * C[j] - 4 * C[j-1] + C[j-2];
	B[0]= 3;
	A[0]= -4;
	D[0]= 1; /* Here D plays the role of Y */
	/* Here we assume  interv2 > 1 */
    }

}

}


void pr_profiles(int MaxIter)
/*=========================*/
{
int j;
static int count = 0; /* For steady=2, count enables to take different
	action below for steady state calculation proper and for the
	subsequent transient calculation */

if(steady && !count++) {
 double C1, C2 = A2 * l2 / A1, C3 = G/D_T;
 C1 = C3 * C2 * h1;
 C2 *= l1 * C3;
 PR"%c\t\tSteady State\n%c\n", CmntC, CmntC);
 PR"%c No. of iterations done = %d\n", CmntC, MaxIter);
 report_max_NR_error(out, CmntC);
 PR"%c  x (cm)      [H_2]    [Analytical steady state]\n", CmntC);
 for(j= 0;j<=interv1;j++) PR" %f  %e  %e\n", j * h1, C_O[j], C1 * j);
 for( ;j<nnodes;j++) {
	C1 = (j - interv1) * h2;
	PR" %f  %e  %e\n", l1 + C1, C_O[j], C3 * C1 * (l2 - 0.5 * C1) + C2);
 }
} else {
 PR"#Time = %e sec = %e hours = %e days\n", t, t/3600., t/86400.);
 PR"#Last tStep = %e sec = %e hours\n", tStepLast, tStepLast/3600.);
 PR"#Max. # of iterations done since last profile = %d\n",MaxIter);
 report_max_NR_error(out, CmntC);
 PR"%c  x (cm)      [H_2]\n", CmntC);
 c0p= c0; c0= C_O[0];
 for(j= 0;j<=interv1;j++) PR" %f  %e\n", j * h1, C_O[j]);
 cl1p= cl1; cl1= C_O[j];
 for( ;j<nnodes;j++)
	PR" %f  %e\n", l1 + (j - interv1) * h2, C_O[j]);
 cl1l2p= cl1l2; cl1l2= C_O[j];
}
(void)fflush(out);
}
