/* File demo/demo3/demo3.c; TRANSIENT V3.0; 6 June 2005 */
/* (C) 1995,2001,2005 Miroslav Kolar */

/* ============================================================ */
/*  A simple model of stochastic surface growth			*/
/*  see: M. Viscek and T. Viscek,				*/
/*       J.Phys. A: Math.Gen. 28 (1995), L311			*/
/*  (Here Neumann boundary conditions are used instead of the	*/
/*  periodic ones used in the above paper.)			*/
/*  Demonstration module No. 3 for TRANSIENT			*/
/* ============================================================ */


#define PDM_H "demo/demo3/demo3.h"
	/* path of demo3.h relative to the TRANSIENT directory */
#include "../../transien.h"
	/*  >>>> READ the information in transien.h of TRANSIENT <<<<	*/
extern int sepProf;
	/* To be able to redefine its default value
	 - not listed in transien.h */ 

#ifdef DEBUG
#include "../../matrixv.h"
#endif

/* The PDM specific input parameters are defined in demo3.h

 Additional info (to the explanations of demo3.h):
 CF and SF can be used to generate compact plots
	Instead of h(x), SF * (h(x) - CF * <h>) is written
	into the output file
	(see also caption of Fig. 1 of the above paper).
*/

/* Other global variables: */
static double
	New, bNew, mNew2,
	*h, *h_o,  /* the dependent variable = surface height */
	*FD, *AJ, *BJ, *DJ, *XJ, *YJ,
	hAv, bcV2;


int get_neqns_etc(void)
/*===================*/
{
title = "			TRANSIENT\n\
		    Demonstration No. 3\n\
	        Stochastic surface growth";
time_units = "";
/* Redefinition of the default values of several control input
   parameters to better suit the current model: */
tEnd = 5000.;
tOutInit = 10.;
tOutMulF = 1.;
nOutOTH = 50;
cDifMxInit = 0.01;
sepProf= 0;
return 1;  /* = # of equations */
}


void generate_grid(void)
/*====================*/
{
nnodes = L + 1;
}


#ifndef RAND_MAX
#define RAND_MAX 32767
#define RAND_WARNING { PE"\nCheck also the definition of RAND_MAX in "__FILE__"\n"); }
#else
#define RAND_WARNING
#endif

void Initialize(double *cp, double *c_op)
/*=====================================*/
{
int j;

h = cp;
h_o = c_op;

if(randInit) {
	double Max = randInitAmpl / RAND_MAX;
	RAND_WARNING
	srand(randSeed);
	for(j=0; j<nnodes; j++) h[j] = h_o[j] = Max * rand();
} else
	for(j=0; j<nnodes; j++) h_o[j]= h[j]= 0.;

/* h_o[0] = h[0] = h_o[L] = h[L] = 0.5 * (h_o[0] + h_o[L]); */
/* for(j=0; j<nnodes; j++) h_o[j]= h[j]= (1.-A) * j; */
}

/**********************************************************/
/* h[j]  - surface height				  */
/**********************************************************/


void initNL3BANDV(double ***LBp, double ***UBp, double *Fp,
	double*Ap, double*Bp, double*Dp)
/*=======================================================*/
{
static double *LB, Lower[2];

*LBp = &LB;
**LBp = Lower;
Lower[0] = 1.,  /* Bound type: constant */
Lower[1] = 0.;  /* Bound value */
/* No upper bound set */

/* F, Jacobians: */
FD= Fp;
AJ= Ap;
BJ= Bp;
DJ= Dp;
XJ= Ap;
YJ= Dp;

}

void auxiliary_parameters_for_evalf(void)
/*=====================================*/
{
New = 1. - Tau;
mNew2 = -2. * New;
bNew = 0.5 * B * New;
if(!bc) bcV2 = 2. * bcValue;
}


void init_tStep_for_evalf(void) { }
/*===============================*/


void evalf(int j)
/*=============*/
{
static double hBarM, hBar0, hBarP, B0;
register double aux, sign;

if(j==0){

#ifdef DEBUG
	show_dvector("h1-5  ",h,0,4);
	show_dvector("h_o1-5",h_o,0,4);
	show_dvector("hLast5",h,L-4,L);
#endif

  /* Assume that the first call to evalf() is with j=0 and
     on each subsequent call j is increased by 1 (THIS IS NOT
     CHECKED HERE!).
     So first do some initializations needed later for j > 0:
  */
	B0 = mNew2 - tStepInv; 
	hBar0 = Tau * h_o[0] + New * h[0];
	hBarP = Tau * h_o[1] + New * h[1];

  /* Now do the x=0 (zero-flux) boundary condition: */
	if(bc) {
	 FD[0] = h[0] - bcValue;
	 BJ[0]= 1.;
	} else {
	 FD[0]= -3. * h[0] + 4. * h[1] - h[2] - bcV2;
	 BJ[0]= -3.;
	 DJ[0]= 4.;
	 XJ[0]= -1.;
	}
} else if(j<L) {

	/* First update hBar's: */
	hBarM = hBar0;
	hBar0 = hBarP;
	hBarP = Tau * h_o[j+1] + New * h[j+1];

	aux = 0.5 * (hBarP - hBarM);
	if(aux > 0.) sign = bNew; else { sign = -bNew; aux = -aux; }
	aux += A;
	FD[0] = hBarM - hBar0 - hBar0 + hBarP - B * log(aux)
				+ (h_o[j] - h[j]) * tStepInv;
	aux = sign / aux;
	AJ[0] = New + aux;
	DJ[0] = New - aux;
	BJ[0] = B0;

} else { /* j==L: zero-flux b.c. */

	if(bc) {
	 FD[0] = h[L] - bcValue;
	 BJ[0]= 1.;
	} else {
	 FD[0]= 3 * h[j] - 4 * h[j-1] + h[j-2] - bcV2;
	 BJ[0]= 3;
	 AJ[0]= -4;
	 YJ[0]= 1;
	}
}

}


void pr_profiles(int MaxIter)
/*=========================*/
{
int j;
static int count = 0; /* For steady=2, count enables to take different
	action below for steady state calculation proper and for the
	subsequent transient calculation */
double aux = CF * hAv;

if(steady && !count++) {
 PR"%c\t\tSteady State\n%c\n", CmntC, CmntC);
 PR"%c No. of iterations done = %d\n", CmntC, MaxIter);
} else {
 PR"%c Time = %e\n", CmntC, t);
 PR"%c Last tStep = %e\n", CmntC, tStepLast);
 PR"%c Max. # of iterations done since last profile = %d\n", CmntC, MaxIter);
}
 report_max_NR_error(out, CmntC);
 PR"%c  x    %g*[h(x)-%g*<h>]\n", CmntC, SF, CF);
 for(j= 0;j<nnodes;j++) PR" %d  %e\n", j, SF * (h_o[j] - aux));
(void)fflush(out);
}


void read_profiles(char *filename)
/*==============================*/
{
PE"\nNo action in read_profiles() implemented\n");
PE"Initializations made in Initialize() unchanged\n\n");
}


void init_Other(void)
/*=================*/
{
Pm"%c\n%c  Surface roughness\n", CmntC, CmntC);
Pm"%c time      w       <h>\n", CmntC);
if(steady) return;
pr_Other(1);
}


int update_Other(void)
/*==================*/
{
/* "Updating" values to be printed in pr_Other
   - nothing to do in this model */
return 0; 
}


void pr_Other(int /*@unused@*/ only_Other)
/*======================================*/
/* In transient.c,
   only_Other = 1  if only pr_Other is called at this time t
                0  if also pr_profiles is called */

/* Here the roughness ("width") of the advancing surface,
   w^2 = <h^2> - <h>^2 is calculated and printed into the par... file
*/
{
int j;
double hAv2, aux;

hAv = 0.;
for(j=0; j<nnodes; j++) hAv += h_o[j];
hAv /= (double)nnodes;
hAv2 = 0.;
for(j=0; j<nnodes; j++) {
	aux = h_o[j] - hAv;
	hAv2 += aux * aux;
}
 
Pm" %g    %g    %g\n", t, sqrt(hAv2/(double)nnodes), hAv);
Fm

}
