/* File demo/demo4/demo4.c; TRANSIENT V3.0; 7 June 2005 */
/* (C) 1995,2001,2005 Miroslav Kolar */

/* ====================================================================	*/
/*                     Brusselator					*/
/*  (Activator-inhibitor reaction-diffusion model for morphogenesis)	*/
/*  Demonstration module No. 4 for  TRANSIENT				*/
/* ====================================================================	*/

#define NEQNS	2

typedef double ROW[NEQNS];

#define PDM_H "demo/demo4/demo4.h"
	/* path of demo4.h relative to the TRANSIENT directory */
#include "../../transien.h"
	/*  >>>> READ the information in transien.h of TRANSIENT <<<<   */

#ifdef DEBUG
#include "../../matrixv.h"
#endif

/* All control input parameters are handled in  transient.c .
   The PDM specific input parameters are defined in  demo4.h .
*/

/* Other global variables: */
static double
	bd, New, bNew, cNew, bdNew, TcNew, h,
	dx, dy, dxNew, dyNew, TdxNew, TdyNew,
	*FD;
static ROW
	*XY, *XY_o,
	*AJ, *BJ, *DJ, *XJ, *YJ;
static int
	bc;


int get_neqns_etc(void)
/*===================*/
{
title = "                     TRANSIENT\n\
                Demonstration No. 4\n\
                    Brusselator";
time_units = "";
/* Redefinition of the default values of several control input
   parameters to better suit the current model: */
tEnd = 50.;
tOutInit = 0.1;
tStepInit = 1e-4;
cDifMxInit = 0.01;
nOutOTH = 1;
/* Convert type of B.C. formula into a numerical value: */
bc= strcmp(BC,"3point");
return NEQNS;
}


void generate_grid(void)
/*====================*/
{
nnodes = interv + 1;
h = L / (double)interv;
}



#ifndef RAND_MAX
#define RAND_MAX 32767
#define RAND_WARNING { PE"\nCheck the definition of RAND_MAX in "__FILE__"\n"); }
#else
#define RAND_WARNING
#endif

void Initialize(double *c_lin, double *c_o_lin)
/*===========================================*/
{
int j,k;
double Max = randInitAmpl / RAND_MAX;

RAND_WARNING
srand(randSeed);

XY = (ROW *)c_lin;
XY_o = (ROW *)c_o_lin;

if(Init) {
	for(j=0; j<nnodes; j++) {
		XY[j][0] = XY_o[j][0] = aa/dd + Max * rand();
		XY[j][1] = XY_o[j][1] = bb * dd / (cc * aa) + Max * rand();
	}
} else
		for(j=0; j<nnodes; j++)
	for(k=0; k<NEQNS; k++) XY[j][k] = XY_o[j][k] = Max * rand();

}


void initNL3BANDV(double ***LBp, double ***UBp, double *Fp,
	double*Ap, double*Bp, double*Dp)
/*=======================================================*/
{
int j;
static double Lower[2], *LB[NEQNS];

*LBp = LB;

Lower[0] = 1.,  /* Bound type: constant */
Lower[1] = 0.;  /* Bound value */
for(j=0; j<NEQNS; j++) LB[j] = Lower; /* All lower bounds are identical */
/* No upper bounds */

FD = Fp;
AJ = (ROW *)Ap;
BJ = (ROW *)Bp;
DJ = (ROW *)Dp;
XJ = AJ;
YJ = DJ;
}


void read_profiles(char *filename)
/*==============================*/
{
PE"\nNo action in read_profiles() implemented\n");
PE"Initializations made in Initialize() unchanged\n\n");
}


/**************************/
/* XY[j][0] = X morphogen */
/* XY[j][1] = Y morphogen */
/**************************/


void auxiliary_parameters_for_evalf(void)
/*=====================================*/
{
New = 1. - Tau;

dx = DX / (h * h);
dy = DY / (h * h);
dxNew = dx * New;
dyNew = dy * New;
TdxNew = 2. * dxNew;
TdyNew = 2. * dyNew;
bd = bb + dd;
bNew = bb * New;
cNew = cc * New;
bdNew = bd * New;
TcNew = 2. * cNew;
}


void reaction_part(double *xy)
/*==========================*/
{
 register double R;

 R = xy[0] * xy[0];
 FD[1] = -(FD[0] = cc * R * xy[1]);
 FD[0] += aa - bd * xy[0];
 FD[1] += bb * xy[0];
 BJ[1][1] = - (BJ[0][1] = cNew * R);
 R = xy[0] * xy[1] * TcNew;
 BJ[0][0] = R - bdNew;
 BJ[1][0] = bNew - R;
}


void init_tStep_for_evalf(void) { }
/*===============================*/


void evalf(int j)
/*=============*/
{
/* Remember, that in the calling procedure, X=A and Y=D.
   All elements of A, B, D are set to zero in nl3band before evalf
   is called. Only their nonzero elements have to be set here. 
*/
static double *cBarM, *cBar0, *cBarP, B00, B11;
static ROW    VM, V0, VP;

if(j==0){

  /* Assume that the first call to evalf() is with j=0 and
     on each subsequent call j is increased by 1 (THIS IS NOT
     CHECKED HERE!).
     So first do some initializations needed later for j > 0:
  */
  { register int i;
    for(i=0; i<NEQNS; i++) {
	V0[i] = Tau * XY_o[0][i] + New * XY[0][i];
	VP[i] = Tau * XY_o[1][i] + New * XY[1][i];
    }
  }
  cBarM = (double *)VM;
  cBar0 = (double *)V0;
  cBarP = (double *)VP;
  B00 = TdxNew + tStepInv;
  B11 = TdyNew + tStepInv;

  /* Now do the x=0 boundary conditions: */
  if(bc) {
   reaction_part(cBar0);
   FD[0] += 2. * dx * (cBarP[0] - cBar0[0]);
   FD[1] += 2. * dy * (cBarP[1] - cBar0[1]);
   /* time derivative: */
   FD[0] += (XY_o[j][0] - XY[j][0]) * tStepInv;
   FD[1] += (XY_o[j][1] - XY[j][1]) * tStepInv;
   BJ[0][0] -= B00;
   DJ[0][0] = TdxNew;
   BJ[1][1] -= B11;
   DJ[1][1] = TdyNew;
  } else {
   FD[0] = -3. * XY[0][0] + 4. * XY[1][0] - XY[2][0];
   BJ[0][0] = -3.;
   DJ[0][0] = 4.;
   XJ[0][0] = -1.;
   FD[1] = -3. * XY[0][1] + 4. * XY[1][1] - XY[2][1];
   BJ[1][1] = -3.;
   DJ[1][1] = 4.;
   XJ[1][1] = -1.;
  }

} else if(j<interv) { /* Internal nodes: */

 /* First update cBar's: */
 { double *p;
   register int i;
   p = cBarM;
   cBarM = cBar0;
   cBar0 = cBarP;
   cBarP = p;
   for(i=0; i<NEQNS; i++)
	cBarP[i] = Tau * XY_o[j+1][i] + New * XY[j+1][i];
 }
   reaction_part(cBar0);
   FD[0] += dx * (cBarM[0] - 2. * cBar0[0] + cBarP[0]);
   FD[1] += dy * (cBarM[1] - 2. * cBar0[1] + cBarP[1]);
   /* time derivative: */
   FD[0] += (XY_o[j][0] - XY[j][0]) * tStepInv;
   FD[1] += (XY_o[j][1] - XY[j][1]) * tStepInv;
   AJ[0][0] = dxNew;
   BJ[0][0] -= B00;
   DJ[0][0] = dxNew;
   AJ[1][1] = dyNew;
   BJ[1][1] -= B11;
   DJ[1][1] = dyNew;

} else { /* j=interv:  x=L boundary conditions */

  if(bc) {
   /* Remember, cBar's were not updated here! */
   reaction_part(cBarP);
   FD[0] += 2. * dx * (cBar0[0] - cBarP[0]);
   FD[1] += 2. * dy * (cBar0[1] - cBarP[1]);
   /* time derivative: */
   FD[0] += (XY_o[j][0] - XY[j][0]) * tStepInv;
   FD[1] += (XY_o[j][1] - XY[j][1]) * tStepInv;
   AJ[0][0] = TdxNew;
   BJ[0][0] -= B00;
   AJ[1][1] = TdyNew;
   BJ[1][1] -= B11;
  } else {
   FD[0] = 3. * XY[j][0] - 4. * XY[j-1][0] + XY[j-2][0];
   BJ[0][0] = 3.;
   AJ[0][0] = -4.;
   YJ[0][0] = 1.;
   FD[1] = 3. * XY[j][1] - 4. * XY[j-1][1] + XY[j-2][1];
   BJ[1][1] = 3.;
   AJ[1][1] = -4.;
   YJ[1][1] = 1.;
  }

}

}


void pr_profiles(int MaxIter)
/*=========================*/
{
int j;
static int count = 0; /* For steady=2, count enables to take different
         action below for steady state calculation proper and for the
         subsequent transient calculation */

if(steady && !count++) {
 PR"%c\t\tSteady State\n%c\n", CmntC, CmntC);
 PR"%c No. of iterations done = %d\n", CmntC, MaxIter);
} else {
 PR"%c Time = %e\n", CmntC, t);
 PR"%c Last tStep = %e\n", CmntC, tStepLast);
 PR"%c Max. # of iterations done since last profile = %d\n",CmntC, MaxIter);
}
report_max_NR_error(out, CmntC);
PR"%c  x         [X]        [Y]\n", CmntC);
for(j= 0;j<nnodes;j++)
	PR" %.8f  %e %e\n", j * h, XY_o[j][0], XY_o[j][1]);
(void)fflush(out);
}


void init_Other(void)
/*=================*/
{ }


int update_Other(void)
/*==================*/
{ return 0; }


void pr_Other(int /*@unused@*/ only_Other)
/*======================================*/
{ }
