/* File doc/PDMtemplate; TRANSIENT V4.3; 31 Oct 2023 */
/* (C) 2001-2005,2017,2023 Miroslav Kolar */

/* Template for the Problem Definition Module source file(s)	*/

/* If PDM consists of N source files (N>=1), the following
   define directive must be present in N-1 of them; it does
   not matter which one is selected as the "first" one that
   does not contain this directive, and which N-1 are the
   "additional" source files:

#define _ADDITIONAL_SOURCE_

See an example in demo/demo2/demo2common.c
*/

#define PDM_H "PDMpath/PDMheader.h"
	/* PDMpath is the path of the PDMheader.h relative to the
		TRD (TRANSIENT Root Directory);
	   PDMheader.h is the name of the PDM header file that goes with
		this source file; header file examples: demo1.h, demo2.h, ...	*/
#include "TRDpath/transien.h"
	/*  TRDpath is the path of the TRD relative to the directory
		where this source file will be located	*/

/* The above two (or three) preprocessor directives are mandatory, as well
   as all the function declarations bellow with the exception of reaction_part().
   However, what is in the body of individual functions is more or less
   just a recommendation (except for the return statement of get_neqns_etc(),
   and the 'nnodes =  ...' line in generate_grid().
   And something to hold the addresses of c, c_o, F, A, ...., Y must
   always be declared.

   The remaining #define directivies below are just examples, not mandatory
*/


#define NEQNS	xxx
	/* xxx  is the number of equations, or unknown variables */

// For example:
#define TA	xxx-1	// a number equal to xxx-1
#define EPS	xxx-2	// a number equal to xxx-2


#if NEQNS!=1
/* Keep in your particular PDM just one branch of all
   #if NEQNS!=1 ... #else ... #endif	blocks
   occuring in this file, according to the respective value of NEQNS,
   which is, of course, constant in a PDM.
*/

typedef double ROW[NEQNS];


/* Declaration of other global variables not listed in PDMheader.h
	(other than the input parameters) will come here */

static ROW *c, *c_o; /* new and old "concentrations" treated
			as 2-dimensional matrices here */

static ROW *AJ, *BJ, *DJ, *XJ, *YJ; /* Jacobian submatrices  - 2-dim */

#else

static double *c, *c_o;
static double *AJ, *BJ, *DJ, *XJ, *YJ;

#endif

static double *FD; /* Discretized F_k */

static double *x; /* grid */


int get_neqns_etc(void)
/*===================*/
{
title = "...";
time_units = "...";
.
.
return NEQNS;
}


void generate_grid(void)
/*====================*/
{
.
nnodes = ...;   /* total number of nodes */
.
x= dvector(nnodes);
.
}


void Initialize(double *c_vec, double *c_o_vec)
/*===========================================*/
{

#if NEQNS!=1
c = (ROW *)c_vec;
c_o = (ROW *)c_o_vec;
#else
c = c_vec;
c_o = c_o_vec;
#endif

...

for(j=0; j<nnodes; j++) {
	c_o[j][0]= c[j][0]= ...;
	c_o[j][1]= ...;
	c[j][1]= ...;
	...
}

}


void initNL3BANDV(double***LBp, double***UBp, double*F,
	double*A, double*B, double*D)
/*===================================================*/
{
/* Declare arrays of pointers to BDA arrays: */
static double *UB[NEQNS], *LB[NEQNS];
/* Or if there are e.g. no upper bounds for all unknowns,
   there is no need to declare UB
   UB[i] points to 1-dim upper BDA array for i-th unknown (equation)
   LB[i] points to 1-dim lower BDA array for i-th unknown (equation)
*/

/* Declare some BDA, for EXAMPLE: */
static double Upper[5], Lower[2];

*LBp = LB;
*UBp = UB;
/* or if there are no upper bounds (no UB): *UBp = NULL */

/* e.g.: */
Lower[0] = 1.;  /* Bound type: constant */
Lower[1] = 0.;  /* Bound value */
Upper[0] = 2.;  /* Bound type: piece-wise constant */
Upper[1] = ...; /* First bound value */
...

for(j=0; j<NEQNS; j++) LB[j] = Lower; /* All lower bounds are identical */

UB[0] = NULL;	// there is no upper bound for 0-th equation
...
UB[3] = Upper;	// piece-wise upper bound for 3-th equation was set above in Upper[]
...

/* Save addresses of F, Jacobian submatrices: */

FD= F;

#if NEQNS!=1
AJ = (ROW *)A;
BJ = (ROW *)B;
DJ = (ROW *)D;
#else
AJ= A;
BJ= B
DJ= D;
#endif
XJ= AJ;
YJ= DJ;

}



#define MAXLETT xxx
	/* xxx should be larger than the length of the longest line + 1 */
#define GETLINE  p= fgets(text,MAXLETT,infile);\
 if(p == NULL) break;\
 if(text[strlen(text)-1] != 10) {\
 PE"\nInput file has lines longer then %d characters!\n", MAXLETT-1);\
exitTR(13,"Please, increase MAXLETT in "__FILE__"!); }


FILE *infile;


void read_profiles(char *filename)
/*==============================*/
/* It is assumed that infile was created by pr_profiles() */
{
 char *p, text[MAXLETT], comnt;
 int j=0, k;

 if( (infile = fopen(filename,"r")) == NULL) {
	PE"Failed to open  %s\n", filename);
	PE"Initializations made in Initialize() unchanged\n");
	return;
 }

 for(;;) {
	GETLINE
	comnt= *p; /* CmntC may differ between runs */
	break;
 }
 do {
	GETLINE
	if(*p != '\n' && *p != comnt) break;
	/* Process file header */
 } while(1);

 do {
	sscanf(text, " %*e %le %le ...", &c_o[j][0], &c_o[j][1], ...);
	j++;
	GETLINE

 } while(1);
 (void)fclose(infile);

  if(!(steady & 1))
	for(j= 0; j<nnodes; j++)
		for(k=0; k<NEQNS; k++) c[j][k]= c_o[j][k];
}


void auxiliary_parameters_for_evalf(void)
/*=====================================*/
{
New = 1. - Tau;

...

}


void init_tStep_for_evalf(void)
/*===========================*/
{
 int mediumM1= initMedium-1;
#ifndef TCONST
 register int m;
#endif
 tMean = t - Tau * tStep;
 tSInvNew = tStepInv / New;

....

}


void reaction_part(double *C)
/*=========================*/
/* My favourite name for a function calculating all reaction
 * terms of all the reaction-diffusion equations;
 * reaction_part() is called repeatedly in evalf() below  */

// Here C is the mixed old-new solution for a single node:
{

#if NEQNS!=1
 FD[0] = ... C[2] ...;
 BJ[0][0] = ...;
 BJ[0][2] = ...;
 ...
#else
 *FD = ...;
 *B = ...;
#endif

}


// NL3BANDV below is a shorthand for "nl3band1.c or nl3bandv.c"

void evalf(int j)
/*===============
Remember, that in the calling function, XJ=AJ and YJ=DJ. All elements
of AJ, BJ, DJ (and thus XJ and YJ) are set to zero in NL3BANDV before
evalf() is called. Thus only their nonzero elements need to be set here.
*/
{
#if NEQNS!=1
static ROW VM, V0, VP, *cBarM, *cBar0, *cBarP;
#else
static double cBarM, cBar0, cBarP;
register k;
#endif
static double ja2, jd2, ...;

if(j<nnodes-1) {
	/* Just an example for the case when the last node at nnodes-1
	   must be treated very differently */

if(j==0){
 /*
 NL3BANDV first calls evalf() with j=0 and
 on each subsequent call j is increased by 1.
 So j=0 is almost always dealt with separately,
 to initialize some things:
 */
#if NEQNS!=1
 for(k=0; k<NEQNS; k++) {
	V0[k] = Tau * c_o[0][k] + New * c[0][k];
	VP[k] = Tau * c_o[1][k] + New * c[1][k];
 }
 cBarM= (double *)VM;
 cBar0= (double *)V0;
 cBarP= (double *)VP;
#else
 cBar0 = Tau * c_o[0] + New * c[0];
 cBarP = Tau * c_o[1] + New * c[1];
#endif
 ...  Some more initializations needed for all j
 ...  ja2, jd2, ... see below

 /* Now do the x=0 boundary conditions: */
 reaction_part(cBar0);
#if NEQNS!=1
 FD[0] += ... ;
 BJ[0][0] += ...;
 BJ[0][2] += ...;
 ...
#else
 *FD += ...;
 *B += ...;
#endif

} else {

// First move the contents of cBar's one grid point:
#if NEQNS!=1
 { double *p;
   p = cBarM;
   cBarM = cBar0;
   cBar0 = cBarP;
   cBarP = p;
   for(k=0; k<NEQNS; k++)
	cBarP[k] = Tau * c_o[j+1][k] + New * c[j+1][k];
 }
#else
 cBarM = cBar0;
 cBar0 = cBarP;
 cBarP = Tau * c_o[j+1] + New * c[j+1];
#endif
 ... update ja2, jd2 for diffusive contribution ...

 reaction_part(cBar0);
  /* Diffusion part:
	Diffusive species are ordered first, DD[k] contain diffusion
	constants for the k-th species: */
#if NEQNS!=1
  for(k=0; k<DIFFUS; k++) {
	  FD[k] +=
	   DD[k] * (ja2*(cBarM[k]-cBar0[k]) + jd2*(cBarP[k]-cBar0[k]));
	  AJ[k][k] = DD[k] * ja2*New;
	  DJ[k][k] = DD[k] * jd2*New;
	  BJ[k][k] -= AJ[k][k] + DJ[k][k];
  }
#else
  *FD += DD * (ja2*(cBarM-cBar0) + jd2*(cBarP-cBar0));
  *AJ = DD * ja2*New;
  *DJ = DD * ja2*New;
  *BJ -= *AJ + *DJ;
#endif

} /* end of if(j==0) */

 /* Time derivatives: */
#if NEQNS!=1
for(k=0; k<NEQNS; k++) {
	F[k] += (c_o[j][k] - c[j][k]) * tStepInv;
	B[k][k] -= tStepInv;
}
#else
*F += (c_o[j] - c[j]) * tStepInv;
*B -= tStepInv;
#endif

} else { /* j=nnodes-1:  last node */


... some special treatment needed, like zero boundary conditions ...


}

}


void pr_profiles(int MaxIter)
/*=========================*/
{
int j, i;

if(MaxIter) {
 if(trueSteady) {
  PR"%c\t\tSteady State results\n%c\n", CmntC, CmntC);
  PR"%c No. of iterations done = %d\n", CmntC, MaxIter);
 } else {
  PR"%c Transient results for t = %.10e sec = %e years\n", CmntC, t, t/Y2S);
  PR"%c Last tStep = %e sec = %.10e years\n", CmntC, tStepLast, tStepLast/Y2S);
  PR"%c Max. # of NR iterations done since the previous profile output = %d\n",
 	CmntC, MaxIter);
 }
 report_max_NR_error(out, CmntC);
} else {
 // MaxIter = 0 indicates that this is the echo of the initial profiles:
 PR"%c\n", CmntC);
}

PR"%c\t\t All concentrations in mol/dm^3\n",CmntC);
PR"%c  x (m)      [S1]      [S2]     ... \n", CmntC);
for(j= 0;j<nnodes;j++) {
	PR" %.10f", x[j]);
	for(i=0; i<NEQNS; i++) PR frmt, C_o[j][i]);
	PR"\n");
}
(void)fflush(out);
}


#define GETVAL(X) {while(*p != ' ') p++; while(*p == ' ') p++; X =  atof(p);}

void init_Other(void)
/*=================*/
{
static count = 0; /* For steady=2, count enables to take different
	 action below for steady state calculation proper and for the
	 subsequent transient calculation */

trueSteady= steady && !count++;
if(trueSteady) return;

...
// Assumning that trans_to_trans was set somewhere above to 1
// if reinitialization was from a transient-run state with
// sepProf=1 and if currently steady=0.

if(init_from_file != NULL && trans_to_trans) {
/* Profiles reinitialized from a file: if not a steady state,
   reinitialization of some quantities for the same time instant
   from the associated .tdep file may be needed/beneficial:
*/

register int i, len = strlen(init_from_file);
register char *p;
char text[MAXLETT], comnt=CmntC, *fileName;
double last_t;
double aux __attribute__((unused));

/* Copy the init_from_file string into a newly created buffer, and
 * replace with 'tdep' the original numerical extension: */
i= len-1;
while(i && init_from_file[i] >= '0' && init_from_file[i] <= '9') i--;
if(init_from_file[i] == '.') len= i;
fileName= makestr(len+5);	// allocates additional byte and fills it with 0
memcpy(fileName,init_from_file,len);
memcpy(fileName+len,".tdep",6);

if((infile = fopen(fileName,"r")) == NULL) {
	if(MAXLETT < 21+len+6) exitTR(98,"MAXLETT needs to be increased!");
	// Because in the following 21+len+6 bytes must be placed into text[]
	memcpy(text,"Failed to open file: ",21);
	memcpy(text+21,fileName,len+6);
	free_str(1,&fileName);
	exitTR(99,text);
}
free_str(1,&fileName);

for(;;) {
	GETLINE
	GETLINE
	comnt= *p;
	break;
}
for(;;) {
	GETLINE
	if(*p != comnt) break;
	/* possibly extract some info from the par file header here */
}
// Here we assume that the profiles file was the last one, corresponding
// to the last entry in the .tdep file (we could instead search for
// another time instant in .tdep, corresponding to the given profiles
// file, that should also be assigned to this new tStart):
//
do {GETLINE} while (p != NULL);
(void)fclose(infile);
p= text; /* = last line */

if(*p != comnt) {
 GETVAL(last_t)
 last_t *= Y2S;
 aux=last_t-tStart;
 if(last_t>1.) aux /= last_t;
 if(fabs(aux)>1e-7) {
  PE"Last time from the old .tdep file, %.16g, is different\nfrom tStart=%.16g\n",
	last_t,tStart);
  exitTR(15,"Time not found in the old .tdep file");
 }
 GETVAL(...)
 GETVAL(...)
 ...
}
/* else is a steady state test ? */

} /* end of init_from_file */

Pm"%c ... \n", CmntC, ...);
Pm"%ctime (years)  ... \n", CmntC, ...);
pr_Other(1);
Fm
}


int update_Other(void)
/*==================*/
{
 ...

 /* e.g.: */
 if(t < ...) {
	pr_Other(1);
	return 1;
 } else return 0;
}


void pr_Other(int only_Other)
/*===========================
 In transient.c, pr_Other is called with:
 only_Other  =  1  if only pr_Other is called at this time t
		0  if pr_profiles is also called at the same time

 One can use the if(only_Other) switch in the body of this
 function e.g. to avoid output duplication, or to send additional
 output from pr_Other into the profiles file using the PR macro.

 On the other hand, if pr_Other output does normally not correspond to the
 end of a time step, but to another time instant (e.g., tMean in demo1.c),
 the if(!only_Other) switch can be used here to generate additional output
 record for the end of the time step to match exactly the time of the
 profiles file. See demo1.c for an example of such a use.
*/
{
	...
	Pm frmt, ...);
	Pm frmt, ...);
	Pm"\n");
	...
	Fm
}

// Note: If only_Other is not used at all in the body of pr_Other,
// use a safer declaration:
// void pr_Other(int /*@unused@*/ only_Other) {...}
