/* File dynarr.c; TRANSIENT V4.3; 20 December 2015
   (C) 2000-2002,2009,2015 Miroslav Kolar

Dynamic array creation/deletion routines
*/
#include <stdlib.h>
#include <stdio.h>

extern char msg[];
extern const int msgLength;
#define PRl (void)snprintf(msg,msgLength,

extern void exitTR(int stat, char *descr) __attribute__ ((noreturn));

/*@-null@*/

void chck_N(int N, const char *function_name)
{
  if(N<1) {
	PRl"%s() was invoked with a non-positive length (# of elements) %d", function_name, N);
	exitTR(-1,msg);
  }
}

void failure_msg(void *p, int n, const char *function_name, const char *what)
{
  if(p == NULL) {
	PRl"%s() failed to allocate a vector of %d %s", function_name, n, what);
	exitTR(-1,msg);
  }
}


/* allocate a vector of pointers to double */
double **pdvector(int nx)
{
  chck_N(nx,"pdvector");
  void *p = malloc(nx*sizeof(double *));
  failure_msg(p,nx,"pdvector","pointers to double");
  return (double **)p;
}


/* allocate a vector of doubles */
double *dvector(int nx)
{
  chck_N(nx,"dvector");
  void *p = malloc(nx*sizeof(double));
  failure_msg(p,nx,"dvector","doubles");
  return (double *)p;
}


/* (re)allocate a vector of doubles */
double *rdvector(double *v, int nx)
{
  chck_N(nx,"rdvector");
  void *p = realloc((void *)v,nx*sizeof(double));
  if(p == NULL) {
	PRl"rdvector() failed to reallocate a vector of %d doubles;\noriginal pointer: %p", nx, v);
	exitTR(-1,msg);
  }
  return (double *)p;
}


/* allocate an int vector */
int *ivector(int n)
{
  chck_N(n,"ivector");
  int *p = (int *)malloc(n*sizeof(int));
  failure_msg(p,n,"ivector","integers");
  return p;
}


/* allocate memory for a string of length n, add 1 char and fill it with the terminal 0 */
char *makestr(int n)
{
  chck_N(n,"makestr");
  char *p = (char *)malloc((n+1)*sizeof(char));
  if(p == NULL) {
	PRl"makestr() failed to allocate memory for a string of %d characters", n);
	exitTR(-1,msg);
  }
  *(p+n)= '\0';
  return p;
}


/* free memory allocated to double vector */
void free_dvector(double **A)
{
    if(*A != NULL) {
	free((void *) *A);
	*A = NULL;
    }
}


/* free memory allocated to int vector */
void free_ivector(int **A)
{
    if(*A != NULL) {
	free((void *) *A);
	*A = NULL;
    }
}


/* free memory allocated to a string */
// by_malloc must be set to 0, if the memory location pointed to by *S
// was not created by malloc
void free_str(short int by_malloc, char **S)
{
    if(by_malloc) {
	if(*S != NULL) {
		free((void *) *S);
		*S = NULL;
	}
    } else *S = NULL;
}

/*@+null@*/
