/* File matinvv.c; TRANSIENT V4.3; 16 December 2015 */
/* (C) 1995,2012,2015 Miroslav Kolar */

// This file has not changed from the very first version in 1995
// (V1.0 or R0) until about 9 January 2012. So it was the same
// for all versions V1.0 through V4.0. It contained the definition
// of matinv() function which was never used all those years.
// Then in V4.1 and V4.2, matinv() was commented out, but not removed.
// Thus actually, the used part of this file has not changed
// since 1995 till Dec. 16, 2015. Then the commented out matinv()
// was finally removed, and also all the accompanied unused MEMSIZE   
// mechanism.

/* Matrix inversion using lu factorization.		*/
/* Adapted from matinv.c by Robert M. Spotnitz, 1990	*/

/* matinvNMC() now pass up the (failure) return 	*/
/* code from  lufac();					*/
/* lusolve() optimized, vector b[] eliminated,		*/
/* all matrices (incl. l, u) linearized (vectorized)	*/

#include <stdio.h>

#include "dynarr.h"
#include "lufacv.h"


static double *x, *l, *u;
static int *nrows;

void cr_matinv_mem(int n) {
/*---------------------*/
    x = dvector(n*(n+2));    // points to a block of memory for x, l and u
    l = x + n;
    u = l + (n*(n+1) >> 1);
    nrows = ivector(n);
} /* end of cr_matinv_mem */

void del_matinv_mem(void) {
/*---------------------*/
    if(x) free_dvector(&x);
    if(nrows) free_ivector(&nrows);
} /* end of del_matinv_mem */


/*  Note: it's OK when  minv  and  a  are identical;
          ========================================= 
     a  may be destroyed (rows reordered) in lufac anyway!
    ==================================================== */
   
int matinvNMC(int n, double *minv, double *a) {
/*-----------------------------------------*/
/* No matinv memory check done */
int i,j;

    if(lufac(n,a,l,u,nrows) < 0) return(-1);
    /* The initial value of x[] doesn't matter: */ 
    for(i=0;i<n;i++) {
        lusolve(n,l,u,nrows,i,x);
        for(j=0;j<n;j++) minv[j * n + i] = x[j];
    };
    return(1);
} /* end of matinvNMC */
