/* File matrixv.c; TRANSIENT V4.3; 10 January 2016 */
/* (C) 1995,2002,2003,2011,2016 Miroslav Kolar */

/* matrix.c - matrix operations */
#include <stdio.h>
#include <math.h>

extern void exitTR(int, char *) __attribute__ ((noreturn));


void mmult(int neqns, double *p, double *l, double *r) {
/*==================================================*/
/*	p = l * r  :   all matrices */
int i,j,k;
register double R;
 for (i=0;i<neqns*neqns;i += neqns)
   for (j=0;j<neqns;j++) {
     R = l[i] * r[j];
     for (k=1;k<neqns;k++) R += l[i+k] * r[k*neqns+j];
     p[i+j] = R;
   }
}  /* end of mmult */


void mvmult(int neqns, double *p, double *m, double *v) {
/*===================================================*/
/*	p = m * v  :   p, v - vectors, m - matrix */
int i, j, k = 0;
register double R;
 for (i=0;i<neqns;i++) {
   R = m[k++] * v[0];
   for (j=1;j<neqns;j++) R += m[k++] * v[j];
   p[i] = R;
 }
}  /* end of mvmult */

void mmultsub(int neqns, double *pd, double *l, double *r) {
/*======================================================*/
/*	pd = pd - l * r  :   all matrices */
int i,j,k;
register double R;
 for (i=0;i<neqns*neqns;i += neqns)
   for (j=0;j<neqns;j++) {
     R = l[i] * r[j];
     for (k=1;k<neqns;k++) R += l[i+k] * r[k*neqns+j];
     pd[i+j] -= R;
   }
}  /* end of mmultsub */

void mvmultsub(int neqns, double *pd, double *m, double *v) {
/*=======================================================*/
/*	pd = pd - m * v  :   pd, v - vectors, m - matrix */
int i, j, k = 0;
register double R;
 for (i=0;i<neqns;i++) {
   R = m[k++] * v[0];
   for (j=1;j<neqns;j++) R += m[k++] * v[j];
   pd[i] -= R;
 }
}  /* end of mvmultsub */

void show_dmatrix(char *name, int n, double *m, int l1, int u1, int l2, int u2, char *frm) {
/*======================================================================================*/
/* n is the number of columns of a matrix stored linearly;
   the format string, frm, must start with a space */
int i,j;
register double ME;
 if(*frm != ' ') exitTR(-1,"frm in a call to show_dmatrix does not start with a space");
 (void)fprintf(stderr,"%s:\n", name);
 for (i=l1;i<=u1;i++) {
	(void)fprintf(stderr,"%d:",i);
	for (j=l2;j<=u2;j++) {
		ME= m[i * n + j];
		if(isnan(ME) || isinf(ME) || ME!=0.) {
			(void)fprintf(stderr, " %d:", j);
			(void)fprintf(stderr, frm, ME);
		}
	}
	(void)fprintf(stderr,"\n");
 }
} /* end of show_dmatrix */

void show_dvector_f(char *name, double *v, int l, int u, char *frm) {
/*===============================================================*/
int i;
 if(*frm != ' ') exitTR(-1,"frm in a call to show_dvector_f does not start with a space");
 (void)fprintf(stderr,"%s:", name);
 for (i=l;i<=u;i++) (void)fprintf(stderr,frm,v[i]);
 (void)fprintf(stderr,"\n");
} /* end of show_dvector */

void show_dvector(char *name, double *v, int l, int u) {
/*==================================================*/
int i;
 (void)fprintf(stderr,"%s:", name);
 for (i=l;i<=u;i++) (void)fprintf(stderr," %.8g",v[i]);
 (void)fprintf(stderr,"\n");
} /* end of show_dvector */

void show_ivector(char *name, int *v, int l, int u) {
/*===============================================*/
int i;
 (void)fprintf(stderr,"%s:", name);
 for (i=l;i<=u;i++) (void)fprintf(stderr," %d",v[i]);
 (void)fprintf(stderr,"\n");
} /* end of show_ivector */
