/* File nl3_conv.h; TRANSIENT V4.3; 3 Nov 2023 */
/* (C) 1999-2016,2018,2019,2023 Miroslav Kolar */

// TOL and TOLC are control input parameter; TOL > 0, TOLC > 0
// Ideal value: TOL=TOLC=1
// Most probably TOL > 1 will be needed
// TOL, TOLC could be < 1 to explore the whole range of precision settings

// FresidPREC is set to TOL * NoNodes * 2e-16,
// CresidPREC = TOLC * (NoEqns-Nexcl) * FresidPREC;
//		Both set in cr_band_mem() and possibly in changeNoNodes().
// cresid is calculated in nl3band?.c as the sum of changes in
//	(NoEqns-Nexcl) * NoNodes elements of c[].

// Maximum practical double precision is about 2e-16 (if two numbers differ
// by less than that, they are indistinguishable most of the time);
// this is used in FresidPREC above.
// Thus if cresid falls below the ideal value of CresidPREC with TOL=TOLC=1, doing
// further NR iterations would most probably just add numerical noise.

// TOL > 1 and TOLC > 1 may be needed almost always to get reasonable execution times

if(monitor2)
	PE"%2d: (%9.4e) %9.4e %3d %2d  (%9.4e) %9.4e %3d %2d\n", NRiter,fresidT,
	fresidM,nodefresidM,eqfresidM,cresidT,cresidM,nodecresidM,eqcresidM);

if(!(finite(fresidT)&&finite(cresidT))) {
	converged= -9.;
	break;
}

if(cresidT<CresidPREC || fresidT<=FresidPREC) {
	// excellent convergence
	converged= 1.;
	break;
}

if(NRiter>1) {
	//if((cresidT>=0.99*cresidT_p || fresidT>=fresidT_p) && NRiter>2) {
	if(NRiter>2 && cresidT>=0.9999*cresidT_p) {
		// cresid does not decrease fast enough any more: either the limit
		// of achievable precision reached, or oscillations started?;
		// and so it's time to stop:
		converged= cresidT/CresidPREC;
		break;
	}
	cresidT_p= cresidT;
}

// converged remains -1 (as set in nl3bandv.c or nl3band1.c) 
